//
//  EmojiArtModel.Background.swift
//  EmojiArt
//
//  Created by Sameh Fakhouri on 11/10/21.
//

import Foundation

extension EmojiArtModel {
    
    enum Background: Equatable, Codable {
        case blank
        case url(URL)
        case imageData(Data)
        
        var url: URL? {
            switch self {
            case .url(let url):
                return url
            default:
                return nil
            }
        }

        var data: Data? {
            switch self {
            case .imageData(let data):
                return data
            default:
                return nil
            }
        }

        enum CodingKeys: String, CodingKey {
            case url = "theURL"
            case imageData = "theImageData"
        }

        init(from decoder: Decoder) throws {
            let container = try decoder.container(keyedBy: CodingKeys.self)
            if let url = try? container.decode(URL.self, forKey: .url) {
                self = .url(url)
            } else if let imageData = try? container.decode(Data.self, forKey: .imageData) {
                self = .imageData(imageData)
            } else {
                self = .blank
            }
        }
        
        func encode(to encoder: Encoder) throws {
            var container = encoder.container(keyedBy: CodingKeys.self)
            switch self {
            case .url(let url):
                try container.encode(url, forKey: .url)
            case .imageData(let data):
                try container.encode(data, forKey: .imageData)
            case .blank:
                break
            }
        }
    }
}
